import task from '../../src/js/markdownCommands/task';
import MarkdownEditor from '../../src/js/markdownEditor';
import EventManager from '../../src/js/eventManager';
import mdListManager from '../../src/js/mdListManager';

describe('task', () => {
    let cm, doc, mde;

    beforeEach(() => {
        const $container = $('<div />');

        $('body').append($container);

        mde = new MarkdownEditor($container, new EventManager());
        mde.componentManager.addManager(mdListManager);

        mde.init();

        cm = mde.getEditor();

        const sourceText = [
            'mytext1',
            '',
            'mytext2',
            'mytext3',
            '',
            '* mytext4',
            '',
            '1. mytext5',
            '# myheading'];

        cm.setValue(sourceText.join('\n'));
        doc = cm.getDoc();
    });

    afterEach(() => {
        $('body').empty();
    });

    describe('add task', () => {
        it('markdown to start', () => {
            doc.setCursor(0, 0);

            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* [ ] mytext1');
        });
        it('Add ol markdown text to line start', () => {
            doc.setCursor(0, 4);

            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* [ ] mytext1');
        });
        it('replace markdown text if line have ul', () => {
            doc.setCursor(2, 4);

            task.exec(mde);

            expect(doc.getLine(2)).toEqual('* [ ] mytext2');
        });
        it('remove task syntax if task already applied', () => {
            doc.setCursor(0, 4);

            task.exec(mde);
            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* mytext1');
        });
        it('빈라인에서 추가된다', () => {
            doc.setCursor(1, 0);

            task.exec(mde);

            expect(doc.getLine(2)).toEqual('* [ ] ');
        });

        it('영역선택후 추가된다', () => {
            doc.setSelection({line: 0, ch: 0}, {line: 2, ch: 7});

            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* [ ] mytext1');
            expect(doc.getLine(1)).toEqual('* [ ] ');
            expect(doc.getLine(2)).toEqual('* [ ] mytext2');
            expect(doc.getLine(3)).toEqual('');
            expect(doc.getLine(4)).toEqual('mytext3');
        });

        it('convert ul to task', () => {
            doc.setCursor(5, 0);

            task.exec(mde);

            expect(doc.getLine(5)).toEqual('* [ ] mytext4');
        });

        it('convert ol to task', () => {
            doc.setCursor(7, 0);

            task.exec(mde);

            expect(doc.getLine(7)).toEqual('1. [ ] mytext5');
        });
        it('Don\'t add task markdown text in heading', () => {
            doc.setCursor(8, 1);

            task.exec(mde);

            expect(doc.getLine(8)).toEqual('# myheading');
        });
        it('add task markdown text except heading', () => {
            const sourceText = ['mytext1', 'mytext2', 'mytext3', 'mytext4', '# myheading'];

            cm.setValue(sourceText.join('\n'));
            doc.setSelection({line: 0, ch: 0}, {line: 4, ch: 7});

            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* [ ] mytext1');
            expect(doc.getLine(1)).toEqual('* [ ] mytext2');
            expect(doc.getLine(2)).toEqual('* [ ] mytext3');
            expect(doc.getLine(3)).toEqual('* [ ] mytext4');
            expect(doc.getLine(4)).toEqual('# myheading');
        });
        it('add task markdown text except blockquote', () => {
            const sourceText = ['mytext1', 'mytext2', 'mytext3', 'mytext4', '> myheading'];

            cm.setValue(sourceText.join('\n'));
            doc.setSelection({line: 0, ch: 0}, {line: 4, ch: 7});

            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* [ ] mytext1');
            expect(doc.getLine(1)).toEqual('* [ ] mytext2');
            expect(doc.getLine(2)).toEqual('* [ ] mytext3');
            expect(doc.getLine(3)).toEqual('* [ ] mytext4');
            expect(doc.getLine(4)).toEqual('> myheading');
        });
        it('add task markdown text except blockquote', () => {
            const sourceText = ['mytext1', 'mytext2', 'mytext3', 'mytext4', '```', 'var a = 10;', '```'];

            cm.setValue(sourceText.join('\n'));
            doc.setSelection({line: 0, ch: 0}, {line: 4, ch: 7});

            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* [ ] mytext1');
            expect(doc.getLine(1)).toEqual('* [ ] mytext2');
            expect(doc.getLine(2)).toEqual('* [ ] mytext3');
            expect(doc.getLine(3)).toEqual('* [ ] mytext4');
            expect(doc.getLine(4)).toEqual('```');
        });
        it('add task markdown text except table', () => {
            const sourceText = ['mytext1', 'mytext2', 'mytext3', 'mytext4', '| hi | hello |', '| --- | --- |', '| bye | bye |'];

            cm.setValue(sourceText.join('\n'));
            doc.setSelection({line: 0, ch: 0}, {line: 4, ch: 7});

            task.exec(mde);

            expect(doc.getLine(0)).toEqual('* [ ] mytext1');
            expect(doc.getLine(1)).toEqual('* [ ] mytext2');
            expect(doc.getLine(2)).toEqual('* [ ] mytext3');
            expect(doc.getLine(3)).toEqual('* [ ] mytext4');
            expect(doc.getLine(4)).toEqual('| hi | hello |');
        });
        it('do not add blank at start & end of task when already blank line exists', () => {
            const sourceText = ['', 'mytext2', '', 'mytext4', '# myheading'];

            cm.setValue(sourceText.join('\n'));
            doc.setCursor(1, 1);
            task.exec(mde);

            expect(doc.getLine(0)).toEqual('');
            expect(doc.getLine(1)).toEqual('* [ ] mytext2');
            expect(doc.getLine(2)).toEqual('');
            expect(doc.getLine(3)).toEqual('mytext4');
            expect(doc.getLine(4)).toEqual('# myheading');
        });
        it('add blank at start & end of task when blank line not exists', () => {
            const sourceText = ['mytext1', 'mytext2', 'mytext3', 'mytext4', '# myheading'];

            cm.setValue(sourceText.join('\n'));
            doc.setCursor(1, 1);
            task.exec(mde);

            expect(doc.getLine(0)).toEqual('mytext1');
            expect(doc.getLine(1)).toEqual('');
            expect(doc.getLine(2)).toEqual('* [ ] mytext2');
            expect(doc.getLine(3)).toEqual('');
            expect(doc.getLine(4)).toEqual('mytext3');
            expect(doc.getLine(5)).toEqual('mytext4');
            expect(doc.getLine(6)).toEqual('# myheading');
        });
        it('add blank at start & end of task when blank line not exists with selection', () => {
            const sourceText = ['mytext1', 'mytext2', 'mytext3', 'mytext4', '# myheading'];

            cm.setValue(sourceText.join('\n'));
            doc.setSelection({line: 1, ch: 0}, {line: 2, ch: 4});
            task.exec(mde);

            expect(doc.getLine(0)).toEqual('mytext1');
            expect(doc.getLine(1)).toEqual('');
            expect(doc.getLine(2)).toEqual('* [ ] mytext2');
            expect(doc.getLine(3)).toEqual('* [ ] mytext3');
            expect(doc.getLine(4)).toEqual('');
            expect(doc.getLine(5)).toEqual('mytext4');
            expect(doc.getLine(6)).toEqual('# myheading');
        });
        it('do not add blank at start & end of task when already blank line exists with selection', () => {
            const sourceText = ['mytext1', '', '* mytext2', '* mytext3', '', 'mytext4', '# myheading'];

            cm.setValue(sourceText.join('\n'));
            doc.setSelection({line: 2, ch: 0}, {line: 3, ch: 4});
            task.exec(mde);

            expect(doc.getLine(0)).toEqual('mytext1');
            expect(doc.getLine(1)).toEqual('');
            expect(doc.getLine(2)).toEqual('* [ ] mytext2');
            expect(doc.getLine(3)).toEqual('* [ ] mytext3');
            expect(doc.getLine(4)).toEqual('');
            expect(doc.getLine(5)).toEqual('mytext4');
            expect(doc.getLine(6)).toEqual('# myheading');
        });
    });
});
